/**
 * main.c
 * Test von 2 FeRAMs CY15B108 am MSP430G2553
 * Die FeRAM-Chips CY15B108 sind FeRAMs mit 8 MBit, entsprechend 1 MByte. Versorgungsspannung: 1,8V..3,6V
 *  Er ist im SO8-SMD-Gehuse erhltlich (und heit dann CY15B108QN-40SXI), daher auch handhabbar fr nicht-industrielle Anwender.
 *  Der Chip kostet zur Zeit (Anfang 2023) etwa 35 Euro
 *
 * 17.02.2023 DL9MWE
 *
 */
#include "main.h"

unsigned int test_write_read(void);
unsigned int check_readback(unsigned char* txData1, unsigned char* rxData, unsigned int len);
void read_uniqueID(unsigned int CS_PIN, unsigned char* uniqueID);
unsigned char read_StatusRegister(unsigned int CS_PIN);

unsigned int main(void)
{
	WDTCTL = WDTPW | WDTHOLD;	    // stop watchdog timer
	vCPU_init();                    // CPU-Clock 16 MHz
	vSPI_init();                    // SPI-Interface
	if(!uiFeRAM_CY15B108_init())    // init and check the presence of two FeRAMs
	   return 0;                    //  failure
	__bis_SR_register(GIE);         // interrupts enabled

    if(!test_write_read())          // Test des Lesens und Schreiben in beide FeRAM mit verschiedenen Konfigurationen
        return 0;                   // Fehler

    // lesen der 8 Bytes der unique Device ID (fr jedes Device unterschiedlich)
    unsigned char uniqueID1[8], uniqueID2[8];
    read_uniqueID(PORTFeRAM_CS1_PIN, uniqueID1);
    read_uniqueID(PORTFeRAM_CS2_PIN, uniqueID2);

    unsigned char statusRegister1, statusRegister2;
    statusRegister1 = read_StatusRegister(PORTFeRAM_CS1_PIN);
    statusRegister2 = read_StatusRegister(PORTFeRAM_CS2_PIN);
    if(statusRegister1 != statusRegister2)
        return 0;
    return 1;
}

unsigned int test_write_read() {
#define text_len    10
    unsigned char *txData1 = "1234567890";
    unsigned char *txData2 = "ABCDEFGHIJ";
    unsigned char *txData3 = "KLMNOPQRST";
    unsigned char rxData[text_len];     // Array fr die zurckgelesenen Daten
    unsigned long FeRAM_txaddr;         // Speicheradresse in den FeRAMs frs Schreiben, Wertebereich 0-FFFFF
    unsigned long FeRAM_rxaddr;         // Speicheradresse in den FeRAMs frs Lesen, Wertebereich dito

    // Test des ersten FeRAM-Chips
    FeRAM_txaddr = 0UL;
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData1, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData2, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData3, text_len);
    FeRAM_rxaddr = 0UL;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData1, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData2, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData3, rxData, text_len))
        return 0;

    // Test des zweiten FeRAM-Chips
    FeRAM_txaddr = FeRAM_maxAddr+1;
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData1, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData2, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData3, text_len);
    FeRAM_rxaddr = FeRAM_maxAddr+1;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData1, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData2, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, 10);
    if(!check_readback(txData3, rxData, text_len))
        return 0;

    // Test des berlaufs zwischen erstem und zweiten FeRAM
    FeRAM_txaddr = FeRAM_maxAddr-5;
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData1, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData2, text_len);
    FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData3, text_len);
    FeRAM_rxaddr = FeRAM_maxAddr-5;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData1, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData2, rxData, text_len))
        return 0;
    FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len);
    if(!check_readback(txData3, rxData, text_len))
        return 0;

    // Test des Erreichens des Speicherendes beim Schreiben
    FeRAM_txaddr = 2*FeRAM_maxAddr-12;
    if((FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData1, text_len)) != 0UL)
        if((FeRAM_txaddr = ulFeRAM_CY15B108_writebytes(FeRAM_txaddr, txData2, text_len)) != 0UL)  // hier muss ein Fehler auftreten
            return 0;

    // .. und beim Lesen
    FeRAM_rxaddr = 2*FeRAM_maxAddr-12;
    if((FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len)) != 0UL)
        if((FeRAM_rxaddr = ulFeRAM_CY15B108_readbytes(FeRAM_rxaddr, rxData, text_len)) != 0UL)  // hier muss ein Fehler auftreten
            return 0;
    return 1;
}

unsigned int check_readback(unsigned char* txData, unsigned char* rxData, unsigned int len) {
    while(len) {
        --len;
        if(rxData[len] != txData[len])
            return 0;
    }
    return 1;
}

// lesen der uniqueID
void read_uniqueID(unsigned int CS_PIN, unsigned char* uniqueID) {
    PORT_SPI_OUT &= ~CS_PIN;                // Start: entspr. CS-Pin auf 0
    ucSPI_Byte(FeRAM_RUID);                 // send command: Read unique Device ID
    vSPI_read_multiplebytes(8, uniqueID);   // read 8 Bytes of answer
    PORT_SPI_OUT |= CS_PIN;                 // fertig: entspr. CS-Pin auf 1
}

// lesen des Status Registers
unsigned char read_StatusRegister(unsigned int CS_PIN) {
unsigned char statusRegister;
    PORT_SPI_OUT &= ~CS_PIN;                // Start: entspr. CS-Pin auf 0
    ucSPI_Byte(FeRAM_RDSR);                 // send command: Read Status Register
    statusRegister = ucSPI_Byte(0);
    PORT_SPI_OUT |= CS_PIN;                 // fertig: entspr. CS-Pin auf 1
    return statusRegister;
}


void vCPU_init(void) {
    // fr MCLK wird der interne Oszillator wird verwendet  => Set DCO to 16 MHz
    DCOCTL = CALDCO_16MHZ;
    BCSCTL1 = CALBC1_16MHZ;
    BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
    P1DIR = P1OUT = 0;                      // Port 1 is not used
    P3DIR = P3OUT = 0;                      // Port 3 ist in der DIP-Version des Prozessors nicht verfgbar: unused
}


// Ende main.c

